@@ -3,7 +3,6 @@ __all__ = ["Spec"]
 
 # standard library
 from dataclasses import (
-    MISSING,
     Field as Field_,
     dataclass,
     field as field_,
@@ -11,12 +10,12 @@ from dataclasses import (
     replace,
 )
 from functools import lru_cache
-from typing import Any, Callable, Hashable, List, Optional, Type
+from typing import Any, Callable, Hashable, List, Literal, Optional, Type
 
 
 # dependencies
-from typing_extensions import Literal, get_type_hints
-from .typing import P, DataClass, Pandas, Role, get_dtype, get_name, get_role
+from typing_extensions import get_type_hints
+from .typing import T, Pandas, Tag, get_dtype, get_name, get_tag
 
 
 # runtime classes
@@ -27,26 +26,26 @@ class Field:
     id: str
     """Identifier of the field."""
 
-    name: Hashable
-    """Name of the field."""
+    tag: Literal["attr", "column", "data", "index"]
+    """Tag of the field."""
 
-    role: Literal["attr", "column", "data", "index"]
-    """Role of the field."""
+    name: Hashable = None
+    """Name of the field data."""
 
-    default: Any = MISSING
+    default: Any = None
     """Default value of the field data."""
 
     type: Optional[Any] = None
-    """Type (hint) of the field data."""
+    """Type or type hint of the field data."""
 
     dtype: Optional[str] = None
     """Data type of the field data."""
 
-    def update(self, obj: DataClass[P]) -> "Field":
-        """Update the specification by a dataclass object."""
+    def update(self, obj: Any) -> "Field":
+        """Update the specification by an object."""
         return replace(
             self,
-            name=format_name(self.name, obj),
+            name=format_(self.name, obj),
             default=getattr(obj, self.id, self.default),
         )
 
@@ -57,25 +56,25 @@ class Fields(List[Field]):
     @property
     def of_attr(self) -> "Fields":
         """Select only attribute field specifications."""
-        return Fields(field for field in self if field.role == "attr")
+        return Fields(field for field in self if field.tag == "attr")
 
     @property
     def of_column(self) -> "Fields":
         """Select only column field specifications."""
-        return Fields(field for field in self if field.role == "column")
+        return Fields(field for field in self if field.tag == "column")
 
     @property
     def of_data(self) -> "Fields":
         """Select only data field specifications."""
-        return Fields(field for field in self if field.role == "data")
+        return Fields(field for field in self if field.tag == "data")
 
     @property
     def of_index(self) -> "Fields":
         """Select only index field specifications."""
-        return Fields(field for field in self if field.role == "index")
+        return Fields(field for field in self if field.tag == "index")
 
-    def update(self, obj: DataClass[P]) -> "Fields":
-        """Update the specifications by a dataclass object."""
+    def update(self, obj: Any) -> "Fields":
+        """Update the specifications by an object."""
         return Fields(field.update(obj) for field in self)
 
 
@@ -86,6 +85,9 @@ class Spec:
     name: Optional[str] = None
     """Name of the specification."""
 
+    origin: Optional[type] = None
+    """Original dataclass of the specification."""
+
     factory: Optional[Callable[..., Pandas]] = None
     """Factory for pandas data creation."""
 
@@ -93,49 +95,32 @@ class Spec:
     """List of field specifications."""
 
     @classmethod
-    def from_dataclass(cls, dataclass: Type[DataClass[P]]) -> "Spec":
+    def from_dataclass(cls, dataclass: type) -> "Spec":
         """Create a specification from a data class."""
         fields = Fields()
 
         for field_ in fields_(eval_types(dataclass)):
-            field = convert_field(field_)
-
-            if field is not None:
+            if (field := get_field(field_)) is not None:
                 fields.append(field)
 
         factory = getattr(dataclass, "__pandas_factory__", None)
-        return cls(dataclass.__name__, factory, fields)
+        return cls(dataclass.__name__, dataclass, factory, fields)
 
-    def update(self, obj: DataClass[P]) -> "Spec":
-        """Update the specification by a dataclass object."""
-        return replace(self, fields=self.fields.update(obj))
+    def update(self, obj: Any) -> "Spec":
+        """Update the specification by an object."""
+        if self.origin is None or isinstance(obj, self.origin):
+            return replace(self, fields=self.fields.update(obj))
+        else:
+            return self.update(self.origin(obj))
 
-    def __matmul__(self, obj: DataClass[P]) -> "Spec":
+    def __matmul__(self, obj: Any) -> "Spec":
         """Alias of the update method."""
         return self.update(obj)
 
 
 # runtime functions
 @lru_cache(maxsize=None)
-def convert_field(field_: "Field_[Any]") -> Optional[Field]:
-    """Convert a dataclass field to a field specification."""
-    role = get_role(field_.type)
-
-    if role is Role.OTHER:
-        return None
-
-    return Field(
-        id=field_.name,
-        name=get_name(field_.type, field_.name),
-        role=role.name.lower(),  # type: ignore
-        default=field_.default,
-        type=field_.type,
-        dtype=get_dtype(field_.type),
-    )
-
-
-@lru_cache(maxsize=None)
-def eval_types(dataclass: Type[DataClass[P]]) -> Type[DataClass[P]]:
+def eval_types(dataclass: Type[T]) -> Type[T]:
     """Evaluate field types of a dataclass."""
     types = get_type_hints(dataclass, include_extras=True)
 
@@ -145,12 +130,38 @@ def eval_types(dataclass: Type[DataClass[P]]) -> Type[DataClass[P]]:
     return dataclass
 
 
-def format_name(name: Hashable, obj: DataClass[P]) -> Hashable:
-    """Format a name by a dataclass object."""
-    if isinstance(name, tuple):
-        return type(name)(format_name(elem, obj) for elem in name)
+def format_(obj: T, by: Any) -> T:
+    """Format a string or nested strings in an object."""
+    tp = type(obj)
 
-    if isinstance(name, str):
-        return name.format(obj)
+    if isinstance(obj, str):
+        return tp(obj.format(by))  # type: ignore
+    elif isinstance(obj, (list, tuple, set)):
+        return tp(format_(item, by) for item in obj)  # type: ignore
+    elif isinstance(obj, dict):
+        return tp(format_(item, by) for item in obj.items())  # type: ignore
+    else:
+        return obj
 
-    return name
+
+@lru_cache(maxsize=None)
+def get_field(field_: "Field_[Any]") -> Optional[Field]:
+    """Create a field specification from a dataclass field."""
+    tag = get_tag(field_.type)
+
+    if tag is Tag.OTHER:
+        return None
+
+    if tag in Tag.DATA | Tag.INDEX:
+        dtype = get_dtype(field_.type)
+    else:
+        dtype = None
+
+    return Field(
+        id=field_.name,
+        tag=tag.name.lower(),  # type: ignore
+        name=get_name(field_.type, field_.name),
+        default=field_.default,
+        type=field_.type,
+        dtype=dtype,
+    )
